﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace Infragitstics.MotionFramework
{
    public abstract class AnimationBase
    {
        public event EventHandler AnimationCompleted;
        protected virtual void OnAnimationCompleted(EventArgs e)
        {
            if (AnimationCompleted != null)
                AnimationCompleted(this, e);
        }
        protected virtual string ResourceKey
        {
            get
            {
                return GetType().FullName;
            }
        }

        protected abstract Storyboard CreateStoryboard();

        protected abstract void ApplyValues(Storyboard storyboard);

        public virtual void Apply(FrameworkElement element)
        {
            if (element == null)
                throw new ArgumentNullException("element");
            Storyboard storyboard = null;
            if (element.Resources.Contains(ResourceKey))
            {
                storyboard = element.Resources[ResourceKey] as Storyboard;
                storyboard.Pause();
            }
            else
            {
                storyboard = CreateStoryboard();
                element.Resources.Add(ResourceKey, storyboard);
                foreach (var timeline in storyboard.Children)
                {
                    Storyboard.SetTarget(timeline, element);
                }
            }
            ApplyValues(storyboard);
            storyboard.Completed += OnStoryboardCompleted;
            storyboard.Begin();
        }

        protected virtual void OnStoryboardCompleted(object sender, EventArgs e)
        {
            OnAnimationCompleted(EventArgs.Empty);
        }
    } 

    public class SizeAnimation : AnimationBase
    {
        public static TimeSpan DefaultDuration = TimeSpan.FromMilliseconds(250);
        public SizeAnimation(double width, double height)
            : this(width, height, DefaultDuration)
        {
        }

        public SizeAnimation(double width, double height, TimeSpan duration)
        {
            Width = width;
            Height = height;
            Duration = duration;
        }

        public double Width
        {
            get;
            private set;
        }

        public double Height
        {
            get;
            private set;
        }

        public TimeSpan Duration
        {
            get;
            private set;
        }

        protected override Storyboard CreateStoryboard()
        {
            var storyboard = new Storyboard();
            var widthAnimation = new DoubleAnimationUsingKeyFrames();
            var heightAnimation = new DoubleAnimationUsingKeyFrames();

            Storyboard.SetTargetProperty(widthAnimation, new PropertyPath("(FrameworkElement.Width)"));
            Storyboard.SetTargetProperty(heightAnimation, new PropertyPath("(FrameworkElement.Height)"));
            storyboard.Children.Add(widthAnimation);
            storyboard.Children.Add(heightAnimation);
            widthAnimation.KeyFrames.Add(new SplineDoubleKeyFrame()
            {
                KeySpline = new KeySpline()
                {
                    ControlPoint1 = new Point(0.528, 0),
                    ControlPoint2 = new Point(0.142, 0.847)
                }
            });

            heightAnimation.KeyFrames.Add(new SplineDoubleKeyFrame()
            {
                KeySpline = new KeySpline()

                {
                    ControlPoint1 = new Point(0.528, 0),
                    ControlPoint2 = new Point(0.142, 0.847)
                }
            });
            return storyboard;
        }

        protected override void ApplyValues(Storyboard storyboard)
        {
            if (storyboard == null)
                throw new ArgumentNullException("storyboard");
            var widthAnimation = storyboard.Children[0] as DoubleAnimationUsingKeyFrames;
            var heightAnimation = storyboard.Children[1] as DoubleAnimationUsingKeyFrames;
            widthAnimation.KeyFrames[0].Value = Width;
            widthAnimation.KeyFrames[0].KeyTime = KeyTime.FromTimeSpan(Duration);
            heightAnimation.KeyFrames[0].Value = Height;
            heightAnimation.KeyFrames[0].KeyTime = KeyTime.FromTimeSpan(Duration);
        }
    }

    public static class AnimationExtensions
    {
        public static void AnimateSize(this FrameworkElement element, double width, double height)
        {
            new SizeAnimation(width, height).Apply(element);
        }

        public static void AnimateSize(this FrameworkElement element, double width, double height, TimeSpan duration)
        {
            new SizeAnimation(width, height, duration).Apply(element);
        } 

    }
}